/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _ASTAR_H_
#define _ASTAR_H_

#include "AStarTile.h"
#include "IsoBomb.h"

/**
 * Finds paths between 2 points using the A* Algorithm.
 */
class AStar {
    
public:

    /**
     * Constructor
     */
    AStar();

    /**
     * Destructor
     */
   ~AStar();

    /**
     * Finds a path between 2 points.
     *
     * @param source                the source point
     * @param dest                  the destination point
     * @returns                     true if a path was found
     */
    bool findpath( const Point& source, const Point& dest );

    /**
     * Returns the next point of the found path.
     *
     * @returns                     the next point of the found path
     */
	Point getNextPoint();

    /**
     * Returns true if there are more points in the found path.
     *
     * @returns                     true if there are more points in the found path
     */
	bool hasMorePoints();

private:

    /**
     * Clears the path and prepares for another run of findpath.
     */
    void clear();

    /**
     * Calculates the manhattan distance between the specified coordinates
     * and the destination point.
     *
     * @param x                     the x coordinate
     * @param y                     the y coordinate
     * @returns                     the manhattan distance
     */
    int getManhattanDistance( int x, int y );

    /**
     * Creates a tile for the specified x,y coordinate and using the specified
     * parent.  Runs the A* algorithm on the new tile.
     */
    void processTile( AStarTile* parent, int x, int y );

private:

    /**
     * Number of tiles on the map in the x direction
     */
    int numTilesX;

    /**
     * Number of tiles on the map in the y direction
     */
    int numTilesY;

    /**
     * The x coordinate of the source tile
     */
    int sourceX;

    /**
     * The y coordinate of the source tile
     */
	int sourceY;

    /**
     * The x coordinate of the destination tile
     */
    int destX;

    /**
     * The y coordinate of the destination tile
     */
	int destY;

    /**
     * Sorted list used to quickly find tiles in the
     * open list.
     */
    std::set<AStarTileKey> sortedList;

    /**
     * The closed list (used to quickly find tiles)
     */
    std::map<Point,AStarTile*> openList;

    /**
     * The open list (used to quickly find tiles)
     */
    std::map<Point,AStarTile*> closedList;

    /**
     * Iterator for the open list
     */
    std::map<Point,AStarTile*>::iterator openIt;

    /**
     * Iterator for the closed list
     */
    std::map<Point,AStarTile*>::iterator closedIt;

    /**
     * Iterator for the sorted list
     */
    std::set<AStarTileKey>::iterator sortedIt;

    /**
     * The path found by running findpath
     */
	std::list<Point> path;

};

#endif