/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "Animation.h"

Animation::Animation()  {
    init();
}

void Animation::init() {
    // Initialize all the variables
    animation = NULL;
    totalTextures = 0;
    currentFrame = 0; 
    frameTime = 0;
    totalAnimationRuntime = 0.5;
    frameLength = 0;
    animStopped = true;    
    offsetX = 0;
    offsetY = 0;
    alphaValue = 255;
    alphaStartValue = 255;
    useAlpha = false;
    alphaSpeed = 1.0;
    red = 255;
    green = 255;
    blue = 255;

     // Allocate memory for the D3DXVECTOR2* and set the amount to translate it
    translateSprite = new D3DXVECTOR2;
    Point pt;
    pt.x = 400;
    pt.y = 300;
    // Sets up the translateSprite vector
    //setPosition( pt );

    // Allocate memory for the D3DXVECTOR2* and set the size to scale it to
    scaleSprite = new D3DXVECTOR2;
    scaleSprite->x = 1.0;
    scaleSprite->y = 1.0;

    // Create the sprite
    if( FAILED( D3DXCreateSprite( g_dx->getDeviceHandle(), &m_sprite ) ) ) {
        TRACE( "Failed to create sprite" );
    }

    // Register the listener, so switching modes will work correctly
    g_dx->registerListener( *this );   
}

Animation::~Animation() {
    // Remove the listener
    g_dx->removeListener( *this );

    // Delete the transformation vectors
    delete scaleSprite;
    delete translateSprite;

    // Release the sprite if it exists
    if( m_sprite ) 
        m_sprite->Release();
}

/**
 * Draws the current position of the character in the animation
 *
 */
void Animation::draw( const Point& anchor ) {
    //Translate so the given Point is the feet.
    translateSprite->y = anchor.y - offsetY;
    translateSprite->x = anchor.x - offsetX;

    m_sprite->Begin();
    m_sprite->Draw( animation->at( currentFrame ), // Texture to draw
                    NULL, // Rectangle of the part of the texture to draw
                    scaleSprite, // Scaling amount
                    0, 0, // Rotation center and rotation amount
                    translateSprite, // Translate amounts
                    D3DCOLOR_RGBA(red, green, blue, alphaValue ) ); // Color
    m_sprite->End();
}

void Animation::update( float dt ) {
    // Only update if the animation is not stopped
    if( !animStopped ) {
        // Add the time since the last frame( dt ) to the current time
        // the animatin has been running
        frameTime += dt;

        // Current frame wil be the current time the animation has been
        // running divided by the length of each frame
        currentFrame = (int)(frameTime / frameLength);

        if( useAlpha ) {
            alphaValue = (int)(alphaStartValue - ( 
                ( alphaStartValue * frameTime / totalAnimationRuntime ) * alphaSpeed ) );
            
            // Should never get here, but just in case this keeps from getting
            // a negative alpha value, which results in the image looking very odd
            if( alphaValue < 0 )
                alphaValue = 0;
        }

        // Animaiton is over if it gets past the final texture, so
        // reset all the relevant variables
        if( currentFrame >= totalTextures ) {
            currentFrame = 0;
            frameTime = 0;
            animStopped = true;
        }       
    }
}

void Animation::start() {
    animStopped = false;
}

void Animation::stop() {
    animStopped = true;
    currentFrame = 0;
	frameTime = 0;
}

int Animation::getNumTextures() {
    return totalTextures;
}

void Animation::setAnimationFrames( AnimationFrames* frames ) {
    // Sets the animation pointer to the frames pointer
    animation = frames;
    // Gets the total number of textures
    totalTextures = animation->size();
    // Sets the framelength
    frameLength = totalAnimationRuntime / totalTextures;
}

AnimationFrames* Animation::getAnimationFrames() {
    return animation;
}

void Animation::setAnimRuntime( float runtime ) {
    // Sets the lenght of the animation
    totalAnimationRuntime = runtime;
    // Sets the framelength since the totalAnimationRuntime changed
    frameLength = totalAnimationRuntime / totalTextures;
}

float Animation::getAnimRuntime() {
    return totalAnimationRuntime;
}

void Animation::OnLostDevice() {
    m_sprite->OnLostDevice();
}

void Animation::OnResetDevice() {
    m_sprite->OnResetDevice();
}

void Animation::setScaleFactor( float factor ) {
    scaleSprite->x = factor;
    scaleSprite->y = factor;
}

void Animation::setOffsets( int x, int y ) {
    offsetX = x;
    offsetY = y;
}

bool Animation::isAnimStopped() {
    return animStopped;
}

void Animation::setUsingAlpha( bool alpha ) {
    useAlpha = alpha;
}

void Animation::setAlphaBlendSpeed( float speed ) {
    alphaSpeed = speed;
}

void Animation::setStartAlphaValue( int a ) {
    alphaStartValue = a;

    // Check to be sure it is a valid value
    if( alphaStartValue > 255 )
        alphaStartValue = 255;

    if( alphaStartValue < 0 )
        alphaStartValue = 0;

    // Set this here so if draw is called before an update, then 
    // the alphaValue will be set correctly
    alphaValue = alphaStartValue;
}

void Animation::setAlphaValue( int a ) {
    alphaValue = a;
}

void Animation::setColor( int r, int g, int b ) {
    red = r;
    green = g;
    blue = b;
}