/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _ANIMATION_H_
#define _ANIMATION_H_

#include "AnimationManager.h"
#include "GameDXListener.h"
#include "IsoBomb.h"

/**
 *  Animation contains a pointer to a current set of animation frames that
 *  are obtained from the animation manager, it uses those to animate the
 *  object( character or bombs ).  The update function is used to keep
 *  the animation running at a constant speed no matter what the framerate
 *  is and it updates the current texture, which is then drawn in the draw
 *  method.  Also implements the GameDXListener, which is used when switching
 *  modes, the device will be lost, and the OnLostDevice and OnREsetDevice
 *  handles those problems.
 */
class Animation : public GameDXListener {

private:
    // Holds the sprite
    LPD3DXSPRITE m_sprite; 

    // Stores x and y values for scaling the sprite
    D3DXVECTOR2* scaleSprite;
    // Stores x and y values for translating the sprite
    D3DXVECTOR2* translateSprite; 
        
    // Total number of textures in the animation
    int totalTextures; 
    // Current frame to be drawn
    int currentFrame; 

    // Total amount of time the animation should take to run
    float totalAnimationRuntime; 
    
    // The current time when running the animation, used to keep track of
    // the current frame
    float frameTime;
    
    // Length of each frame, frameLength = totalAnimationRuntime / totalTextures 
    float frameLength;

    // Keeps track if the animation is running or not, so updates don't happen
    bool animStopped;

    // A pointer to the current set of animation frames
    AnimationFrames* animation;

    // The x and y offsets
    int offsetX;
    int offsetY;

    // Value used to for alpha blending
    int alphaValue;

    // The value alphaStarts at for blending
    int alphaStartValue;

    // False means do not use alpha
    bool useAlpha;

    // Value to control how fast the alpha decreases
    float alphaSpeed;

    /**
     * RGB values for drawing
     */
    int red, green, blue;

public:

    /**
     *  Draws the current position of the character in the animation
     *
     * @param anchor                point to attach
     */
    void draw( const Point& anchor );

    /**
     *  Updates the animation to the next frame and keeps track of when
     *  the animation should have ended.  Used the time since the last frame
     *  to know when to move onto the next frame based on the length of each
     *  frame.
     *
     *  @params dt The time since the last frame
     */
    void update( float dt );

    /**
     *  Returns the nubmer of textures in the animation
     *  
     *  @returns Total number of textures
     */
    int getNumTextures();

    /**
     *  Returns the current animation frames
     *
     *  @returns a pointer to the current animation frames
     */
    AnimationFrames* getAnimationFrames();

    /**
     *  Sets the current animation frames
     *
     *  @params frames the pointer to the new animation frames
     */
    void setAnimationFrames( AnimationFrames* frames );
    
    /**
     *  Returns the runtime for the animation
     *
     *  @returns the runtime
     */
    float getAnimRuntime();

    /**
     *  Sets the animation runtim
     *
     *  @params runtime The total time the animation taeks to run
     */
    void setAnimRuntime( float runtime );

    /**
     *  Sets the scale factore of the sprite
     *
     *  @params factor the factor to scale the sprite by
     */
    void setScaleFactor( float factor );

    /**
     *  Stops the animation, animStopped = true
     *
     */
    void stop();

    /**
     *  Starts the animation, animStopped = false
     * 
     */
    void start();

    /**
     *  Sets the offset of the image
     *
     *  @params x the x offset
     *  @params y the y offset
     */
    void setOffsets( int x, int y );

    /**
     *  Returns true if the animation has finished, used so explosion 
     *  entities will be destroyed
     *
     *  @returns True if explosion is over
     */
    bool isAnimStopped();

    /**
     *  Turns alpha blending on or off
     *
     *  @params alpha True to use alpha blending
     */
    void setUsingAlpha( bool alpha );

    /**
     *  1.0 would be the alpha goes from its starting value to 0 over the
     *  course of all the frames in the animation.
     *
     *  @params speed the speed to blend at
     */
    void setAlphaBlendSpeed( float speed );

    /**
     *  Sets the starting value from alpha, it defaults to 255, which is no
     *  alpha, can be set to anything from 0 - 255
     *
     *  @params a The new value to start alpha blending from
     */
    void setStartAlphaValue( int a );

    /**
     * Sets an alpha value to apply to all frames of the animation.
     *
     * @param a                 alpha value
     */
    void setAlphaValue( int a );

    /**
     * Sets the color to use when drawing the sprite.
     *
     * @param r                 red value
     * @param g                 green value
     * @param b                 blue value
     */
    void setColor( int r, int g, int b );

    // Constructors
    Animation();

    // Destructor
    ~Animation();

public:
    /**
     *  Inititalizes the animation object
     *
     */
    void init();

public:
    /**
     * Event called when device is lost.
     */
    virtual void OnLostDevice();

    /**
     * Event called when device is reset.
     */
    virtual void OnResetDevice();

};

#endif
