/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _ANIMATIONMANAGER_H_
#define _ANIMATIONMANAGER_H_

#include "IsoBomb.h"

/**
 *  The animationManager loads all the textures that are going to be used for
 *  animations in the game.  It is used by retrieving a pointer to different
 *  sets of animations and then those are set to the current animation, in the
 *  animation class.  All you have to do to change the animation for the
 *  character, is update the pointer to the fames for a different animation.
 *
 */
class AnimationManager {

public:

    /**
     *  Initializes all the textures by calling loadAllGraphics.
     *  The separate function is there in case the graphics ever
     *  need to be reloaded.
     *
     */
    AnimationManager();

    /**
     *  Destructor
     *
     */
    ~AnimationManager();

    /**
     *  Uses vectors to load all the graphics needed for the game,
     *  this should only need to be called in the constructor, but
     *  if something causes the graphics too be lost, we can restore
     *  them with this method
     *
     *  @returns true if successful
     */

    bool loadAllGraphics();
    /**
     *  Used to return the different animation frames for the character,
     *  currently tthere are 4 different sets, 1 for each direction and
     *  the direction sent should be a constant which is defined in 
     *  Isobomb.h
     *
     *  @param direction    A constant value from Isobomb.h
     *
     *  @returns A pointer to the animation frames
     */
    AnimationFrames* getCharacterAnimationFrames( int direction );

    /**
     *  Returns the bomb animation. 
     *
     *  @param direction    A constant value from Isobomb.h
     *
     *  @returns A pointer to the animation frames
     */
    AnimationFrames* getBombAnimationFrames( int direction );

    /**
     *  Returns the bombfeet animation.
     *
     *  @param direction    A constant value from Isobomb.h
     *
     *  @returns A pointer to the animation frames
     */
    AnimationFrames* getBombFeetAnimationFrames( int direction );  

    /**
     *  Returns the explosion animation.
     *
     */
    AnimationFrames* getExplosionAnimationFrames();    

	/**
     *  Returns the powerup graphic for hit points
	 *
     */
    AnimationFrames* getHitPointTexture();    

	/**
     *  Returns the powerup graphic for homing bombs
	 *
     */
    AnimationFrames* getHomingBombTexture();    

	/**
     *  Returns the powerup graphic for number of bombs you can throw
	 *
     */
    AnimationFrames* getBombPowerUpTexture(); 

	/**
     *  Returns the powerup graphic for increasing max power
	 *
     */
    AnimationFrames* getPowerPowerUpTexture(); 

	/**
     *  Returns the powerup graphic for increasing blast radius
	 *
     */
    AnimationFrames* getRadiusPowerUpTexture(); 

	/**
     *  Returns the powerup graphic for increasing speed
	 *
     */
    AnimationFrames* getSpeedPowerUpTexture(); 
	
private:
    
    // Stores the current texture that is being loaded, before it 
    // is pushed onto the appropriate vector
    Texture texture;

    // Vectors to store the different animations
    AnimationFrames upLeft;
    AnimationFrames upRight;
    AnimationFrames downLeft;
    AnimationFrames downRight;
    AnimationFrames bombsDownLeft;
    AnimationFrames bombsDownRight;
    AnimationFrames bombsUpLeft;
    AnimationFrames bombsUpRight;
    AnimationFrames feetDownLeft;
    AnimationFrames feetDownRight;
    AnimationFrames feetUpRight;
    AnimationFrames feetUpLeft;
    AnimationFrames explosion;
	AnimationFrames hitPointPowerUp;
	AnimationFrames homingBombPowerUp;
	AnimationFrames bombPowerUp;
	AnimationFrames powerPowerUp;
	AnimationFrames radiusPowerUp;
	AnimationFrames speedPowerUp;
};

#endif