/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "Bomb.h"
#include "Explosion.h"
#include "Game.h"
#include "SoundSystem.h"
#include "Tile.h"
#include "Map.h"

#include "Vector3D.h"
#include "Point3D.h"
#include "Particle.h"

const int NUM_PARTICLES = 40;
const float NUM_FUSE_PARTICLES_PER_SEC = 100.0f;

Bomb::Bomb() {
}

Bomb::Bomb( const Rect& initPos, int initDir, float size, float delay ) : 
    Entity(initPos, initDir), blastSize(size), explodeTime(delay), height(-1.0f),
    pSoFar( 0 ), timeLived( 0.0f ) {

    anim.setScaleFactor( 0.6f );
    anim.setOffsets( 20, 62 );
    anim.setAnimationFrames(g_am->getBombAnimationFrames(initDir));
    anim.setAnimRuntime( delay );
    anim.start();
}

Bomb::~Bomb() {
}

bool Bomb::update( float dt ) {	
    anim.update( dt );

    timeLived += dt;

    if( timeLived >= explodeTime ) {
        blowUp();

        g_snd->playExplosion();

        return true;

    } else {
        //spawn "fuse" particles
        int numSpawn = (int)( NUM_FUSE_PARTICLES_PER_SEC * timeLived ) - pSoFar;
        pSoFar += numSpawn;
        for ( int i = 0; i < numSpawn; ++i ) {
            Particle* p = new Particle(
                Point3D( pos.x, pos.y, getHeight() + 1.0f ),
                Vector3D( getrndf( 0.2f, -0.2f ), getrndf( 0.2f, -0.2f ), getrndf( 10.0f, 3.0f ) ),
                getrndf( 1.5f, 1.0f ) );
            p->setColor( D3DCOLOR_ARGB( 255, 255, 255, getrnd( 255 ) ) );
            p->setSize( 1.0f );
            p->setBounceFactor( 0.0f );
            g_game->addEffect( p );
        }
    }

    return false;
}

void Bomb::draw( const Point& anchor ) {
    anim.draw( anchor );
}

void Bomb::blowUp() {  
    Rect r;
    const Tile* t;
    float centerHeight;
    bool upStopped, downStopped, leftStopped, rightStopped;

    int top = (int)pos.y;
    int left = (int)pos.x;
    r.init( (float)left, (float)top, 0.99f, 0.99f );
    Explosion* explosion = new Explosion( r, height );
    g_game->addEntity( explosion );

    //Generate ourselves some bitchin' particles!
    Point center = pos.getCenter();
    Point3D startPoint( center.x, center.y, getHeight() + 2.0f );
    //thrust some out in each of the 4 directions
    //positive y axis
    int c;
    Particle* p;
    for ( c = 0; c < NUM_PARTICLES; ++c ) {
        p = new Particle(
            startPoint,
            Vector3D( getrndf( 1.0, -1.0 ), getrndf( 7.0f, 3.0f ), getrndf( 2.5f, 1.0f ) ),
            getrndf( 2.0f, 1.0f ) );
        p->setColor( D3DCOLOR_ARGB( 160, 255, getrnd( 255, 128 ), 32 ) );
        g_game->addEffect( p );
    }
    //negative y axis
    for ( c = 0; c < NUM_PARTICLES; ++c ) {
        p = new Particle(
            startPoint,
            Vector3D( getrndf( 1.0, -1.0 ), getrndf( -3.0f, -7.0f ), getrndf( 2.5f, 1.0f ) ),
            getrndf( 2.0f, 1.0f ) );
        p->setColor( D3DCOLOR_ARGB( 160, 255, getrnd( 255, 128 ), 32 ) );
        g_game->addEffect( p );
    }
    //positive x axis
    for ( c = 0; c < NUM_PARTICLES; ++c ) {
        p = new Particle(
            startPoint,
            Vector3D( getrndf( 7.0f, 3.0f ), getrndf( 1.0, -1.0 ), getrndf( 2.5f, 1.0f ) ),
            getrndf( 2.0f, 1.0f ) );
        p->setColor( D3DCOLOR_ARGB( 160, 255, getrnd( 255, 128 ), 32 ) );
        g_game->addEffect( p );
    }
    //negative x axis
    for ( c = 0; c < NUM_PARTICLES; ++c ) {
        p = new Particle(
            startPoint,
            Vector3D( getrndf( -3.0f, -7.0f ), getrndf( 1.0, -1.0 ), getrndf( 2.5f, 1.0f ) ),
            getrndf( 2.0f, 1.0f ) );
        p->setColor( D3DCOLOR_ARGB( 160, 255, getrnd( 255, 128 ), 32 ) );
        g_game->addEffect( p );
    }
    //Now let's spew some out the top!
    for ( c = 0; c < NUM_PARTICLES; ++c ) {
        p = new Particle(
            startPoint,
            Vector3D( getrndf( 1.5f, -1.5f ), getrndf( 1.5f, -1.5f ), getrndf( 15.0f, 10.0f ) ),
            getrndf( 3.0f, 2.0f ) );
        p->setColor( D3DCOLOR_ARGB( 160, 255, getrnd( 255, 128 ), 32 ) );
        g_game->addEffect( p );
    }

    upStopped = downStopped = leftStopped = rightStopped = false;
    t = g_game->getMap()->getTileAt( Point( pos.x, pos.y ) );
    //centerHeight = (float)t->height;
    centerHeight = height;

    // First checks if a tile location is outside the map, if it is, then
    // it doesn't add that explosion, then it tests if the height of the
    // center of the explosion is higher than the tile it is hitting, if
    // it is, then it draws it at the height of that center tile, otherwise
    // it hit a wall and thus the explosion in that direction is stopped,
    // done by the bool being set to true

    for( int i = 1; i <= blastSize; i++ ) {	

        if( !upStopped ) {
            r.init( (float)(left + i), (float)top, 0.99f, 0.99f );
            if( !g_game->getMap()->isLocationOutside( r ) ) {

                t = g_game->getMap()->getTileAt( Point( pos.x + i, pos.y ) );
                if( centerHeight >= t->height ) {
                    explosion = new Explosion( r, centerHeight );
                    g_game->addEntity( explosion );
                } else
                    upStopped = true;
            }
        }

        if( !rightStopped ) {
            r.init( (float)left, (float)(top - i), 0.99f, 0.99f );
            if( !g_game->getMap()->isLocationOutside( r ) ) {

                t = g_game->getMap()->getTileAt( Point( pos.x, pos.y - i) );
                if( centerHeight >= t->height ) {
                    explosion = new Explosion( r, centerHeight );
                    g_game->addEntity( explosion );
                } else
                    rightStopped = true;
            }
        }

        if( !downStopped ) {
            r.init( (float)(left - i), (float)top, 0.99f, 0.99f );
            if( !g_game->getMap()->isLocationOutside( r ) ) {

                t = g_game->getMap()->getTileAt( Point( pos.x - i, pos.y) );
                if( centerHeight >= t->height ) {
                    explosion = new Explosion( r, centerHeight );
                    g_game->addEntity( explosion );
                } else
                    downStopped = true;
            }
        }

        if( !leftStopped ) {
            r.init( (float)left, (float)(top + i), 0.99f, 0.99f );
            if( !g_game->getMap()->isLocationOutside( r ) ) {

                t = g_game->getMap()->getTileAt( Point( pos.x, pos.y + i) );
                if( centerHeight >= t->height ) {
                    explosion = new Explosion( r, centerHeight );
                    g_game->addEntity( explosion );
                } else
                    leftStopped = true;
            }
        }
    }

    g_snd->playExplosion();	
}

float Bomb::getHeight() const {
    return height;
}

void Bomb::doCollision( Entity* otherEntity ) {
    Explosion* e = dynamic_cast<Explosion*>(otherEntity);
    if( e != NULL ) {
        explodeTime = 0.0f;
    }
}
