/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _CHARACTER_H_
#define _CHARACTER_H_

#include "IsoBomb.h"
#include "CharacterProps.h"
#include "Entity.h"
#include "GameDXListener.h"

class ConfigFile;
class Virus;

/**
 * Represents a player or computer controller entity
 * that can be moved on the screen and throw bombs.
 */
class Character : public Entity, GameDXListener {

public:

    /**
     * Initializes and static variables for Character from the ConfigFile.
     */
    static void initVars( ConfigFile& f );

    /**
     * Default constructor
     */
    Character();

    /**
     * Constructor
     *
     * @param initPos               the initial position of the character on the map
     * @param initDir               the initial direction
     * @param wins                  number of times this character has won
     */
    Character( const Rect& initPos, int initDir, int wins );

    /**
     * Destructor
     */
    virtual ~Character();

    /**
     * Applies the affects of the given character properties by adding
     * each property to the character's current properties.  Returns true
     * if the properties were applied.  If after the application any of the 
     * properties goes above the limit, false is returned to indicate that
     * the properties were not applied.
     *
     * @param modprops              the props used to modify the current props
     * @returns                     true if the properties were applied
     */
    bool affectMe( const CharacterProps& modprops );

    /**
     * Attaches a virus to the character.
     *
     * @param v                     a virus
     */
    void affectMe( Virus* v );

    /**
     * Returns the virus currently affecting this player or NULL if there isn't one.
     *
     * @returns                     virus currently affecting this player
     */
    Virus* getVirus() const;

    /**
     * Cause the character to start charging to throw a bomb.
     *
     * @param dt                    the amount of time since the last frame
     */
    void chargeThrow( float dt );

    /**
     * Cause the character to throw a bomb.
     */
    void throwBomb();

    /**
     * Moves the character on the map the given direction.
     *
     * @param dir                   the direction to move
     * @param dt                    the amount of time since the last frame
     */
    void move( int dir, float dt );

    /**
     * If the move method is not called, this method should be called, when no
     * move is made.  For each logical update of the Character, either move or
     * noMove should be called.  Calling both or neither is not allowed.
     *
     * @param dt                    the amount of time since the last frame
     */
    void noMove( float dt );

    /**
     * Draws the character.  The anchor is the center of the player's feet.
     */
    void draw( const Point& anchor );

    /**
     * Update the character's logic and animation.
     *
     * @param dt                    the amount of time since the last frame
     * @returns                     true if the character has died
     */
    bool update( float dt );

    /**
     * Sets the character's properties to the game default.
     */
    void resetProps();

    /**
     * Stops this character's animation.
     */
    void stopAnimation();

    /**
     * Returns true if the chracter is charging up to throw a bomb.
     *
     * @returns                     true if the chracter is charging up to throw a bomb 
     */
    bool isCharging() const;

    /**
     * Lays a homing bomb if the character has any available.
     */
    void homingBomb();

    /**
     * Sets the character homing bombs will target.
     *
     * @param c                     the character homing bombs will target
     */
    void setEnemy( const Character* c );

    /**
     * Called by ThrownBomb when the bomb explodes.  Subtracts from the bomb counter
     * so the player can lay more bombs.
     *
     */
    void decBombCounter();

	/**
	 *	Removes one hitpoint from the character.
	 *
	 */
	void decHitPoints();

    /**
     * Draws a HUD for this player starting at the specified anchor.
     */
	void drawHUD( const Point& anchor );

    /**
     * Returns the current height.
     *
     * @returns                         the current height 
     */
    virtual float getHeight() const;

    /**
     * Moves the character back to its previous position.
     */
    void moveBack();

	/**
	 *	Handles the collision between 2 entities by finding what type of
	 *	entities they are and then handling the different types of collisions;
	 *
	 *  @param otherEntity       the entity to check collision with
	 */
    virtual void doCollision( Entity* otherEntity );

    /**
     * Returns true if the character has died.
     *
     * @returns                  true if the character has died
     */
    bool isDead() const;

    /**
     * Returns this character's props.
     *
     * @returns                 this charcter's props
     */
    CharacterProps getProps() const;

    /**
     * Shrinks or unshrinks the character.  If the flag is true, then
     * the character will be shrunk, otherwise it will be restored to 
     * normal size.
     *
     * @param shrink            if true, character will shrink
     */
    void setShrink( bool shrink );

    /**
     * Returns true if this entity is colliding with the
     * given entity.
     *
     * @param otherEntity       the entity to check collision with
     * @returns                 true if there is a collision
     */    
    virtual bool checkCollision( const Entity* otherEntity );

public:
    /**
     * Event called when device is lost.
     */
    virtual void OnLostDevice();

    /**
     * Event called when device is reset.
     */
    virtual void OnResetDevice();

private:

    static float invincibleTime;

    /**
     * The character's properties.
     */
    CharacterProps props;

    /**
     * The number of bombs laid by the character.
     */
    int numBombs;

    /**
     * The amount of power to use when throwing.
     */
    float power;

    /**
     * The character that will be targeted by homing bombs.
     */
    const Character* enemy;

    /**
     * The sprite used to draw the HUD.
     */
    LPD3DXSPRITE hudSprite;

    /**
     * Invincibility counter when damaged
     */
    float invincibleTimer;

    /**
     * The position before the last update.
     */
    Rect lastPos;

    /**
     * Returns true if the Character has died.
     */
    bool dead;

	float spinSpeed;
	float totalTimeSinceDeath;
	float spinDirection;

    /**
     * The number of times the Character has won a game.
     */
    int numWins;

    /**
     * The virus currently affecting this character or NULL if there is none
     */
    Virus* virus;

};

#endif