/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _ENTITY_H_
#define _ENTITY_H_

#include "IsoBomb.h"
#include "Animation.h"
#include "Drawable.h"

/**
 * A game object that is drawn and can affect gameplay.
 */
class Entity : public Drawable {

public:

    /**
     * Default constructor
     */
    Entity();

    /**
     * Constructor
     *
     * @param initPos           the initial position
     * @param initDir           the initial direction
     */
    Entity( const Rect& initPos, int initDir );

    /**
     * Destructor
     */
    virtual ~Entity();

    /**
     * Returns true if this entity is colliding with the
     * given entity.
     *
     * @param otherEntity       the entity to check collision with
     * @returns                 true if there is a collision
     */    
    virtual bool checkCollision( const Entity* otherEntity );

    /**
     *	Handles the collision between 2 entities by finding what type of
     *	entities they are and then handling the different types of collisions;
     *
     *  @param otherEntity       the entity to check collision with
     */
    virtual void doCollision( Entity* otherEntity );

    /**
     * Update the game logic.
     *
     * @param dt                the amount of time since the last frame
     * @returns                 true if this entity should be removed from the game
     */
    virtual bool update( float dt )= 0;

    /**
     * Draw this entity on the screen, given the anchor point on the screen,
     * which is defined as the center of the Entity's contact with the ground.
     */
    virtual void draw( const Point& anchor ) = 0;

    /**
     * Get the current position on the map.
     *
     * @returns                 the current position on the map
     */
    virtual const Rect& getPosition() const;

    /**
     * Get the direction this Entity is currently facing.
     *
     * @returns                 the direction this Entity is currently facing
     */
    int getDirection() const;

    /**
     * Reverses the direction the entity is facing.
     */
    void reverseDirection();

protected:

    /**
     * The current position on the map.
     */
    Rect pos;

    /**
     * The animation responsible for drawing.
     */
    Animation anim;

    /**
     * The direction this entity is currently facing.
     */
    int direction;

    /**
     * Height value for use by children who may want to cache their heights.
     */
    float cachedHeight;

};

#endif