/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "IsoBomb.h"
#include "GameDXListener.h"
#include "IsoException.h"

//constructor
GameDX::GameDX() {
    //initialize all member pointers to zero
    m_pd3dFont = 0;
    m_pd3dDevice = 0;
    m_pd3d = 0;
    m_pdi = 0;
}

GameDX::~GameDX() {
    //release everything and return all pointers to zero
    releaseD3D();
    releaseDI();

    assert( listeners.empty() );
}

void GameDX::releaseD3D() {
    if (m_pd3dFont) {
        TRACE("Killing D3D Font");
        m_pd3dFont->Release();
        m_pd3dFont = 0;
    }
    if (m_pd3dDevice) {
        TRACE("Killing D3D Device");
        m_pd3dDevice->Release();
        m_pd3dDevice = 0;
    }
    if (m_pd3d ) {
        TRACE("Killing D3D COM Object");
        m_pd3d->Release();
        m_pd3d = 0;
    }
}

HRESULT GameDX::resetD3D() {
    HRESULT r = 0;

    r = setupPresentParams( (g_win->getWindowed() == TRUE) );
    if ( FAILED(r) )
        return r;

    m_pd3dFont->Release();
    generateLostEvent();

    r = m_pd3dDevice->Reset( &d3dpp );
    if ( FAILED(r) )
        return r;
    r = m_pd3dDevice->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
    assert( SUCCEEDED( r ) );

    generateResetEvent();

    //Get a font to draw with as neccessary
    r = D3DXCreateFont(
        m_pd3dDevice,
        (HFONT)GetStockObject(DEVICE_DEFAULT_FONT),
        &m_pd3dFont);
    if( FAILED(r) ) {
        TRACE("Error Creating Font");
        return r;
    }

    return S_OK;
}

void GameDX::releaseDI() {
    if (m_pdi ) {
        TRACE("Killing DirectInput COM Object");
        m_pdi->Release();
        m_pdi = 0;
    }
}

HRESULT GameDX::setupDI() {
    // Initialize DirectInput
    return DirectInput8Create( g_win->get_hInstance(), DIRECTINPUT_VERSION, 
        IID_IDirectInput8, (void**)&m_pdi, NULL);
}

HRESULT GameDX::setupD3D() {
    HRESULT r = 0;

    m_pd3d = Direct3DCreate8( D3D_SDK_VERSION );
    if(m_pd3d == 0)
        throw IsoException("GameDX: Could not create Direct3D Interface");

    r = setupPresentParams( (g_win->getWindowed() == TRUE) );
    if ( FAILED(r) )
        return r;

    //get a handle to the device (watch out if you have two...)
    r = m_pd3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, g_win->getHwnd(), 
        D3DCREATE_HARDWARE_VERTEXPROCESSING, &d3dpp, &m_pd3dDevice);

    if( FAILED(r) )
        r = m_pd3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, g_win->getHwnd(), 
            D3DCREATE_MIXED_VERTEXPROCESSING, &d3dpp, &m_pd3dDevice);

    if( FAILED(r) )
        r = m_pd3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, g_win->getHwnd(), 
            D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &m_pd3dDevice);

    if( FAILED(r) )
        throw IsoException("GameDX: Could not create Direct3D Device");

    //clear the back buffer and then present
    m_pd3dDevice->BeginScene();
    m_pd3dDevice->Clear( 0, 0, D3DCLEAR_TARGET, D3DCOLOR_XRGB(0, 0, 0), 0.0f, 0);
    r = m_pd3dDevice->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
    assert( SUCCEEDED( r ) );

    m_pd3dDevice->EndScene();
    m_pd3dDevice->Present(NULL, NULL, NULL, NULL);    

    //Get a font to draw with as neccessary
    r = D3DXCreateFont(
        m_pd3dDevice,
        (HFONT)GetStockObject(DEVICE_DEFAULT_FONT),
        &m_pd3dFont);
    if( FAILED(r) ) {
        TRACE("Error Creating Font");
        return r;
    }

    return S_OK;
}

LPDIRECT3DDEVICE8 & GameDX::getDeviceHandle() {
    return m_pd3dDevice;
}

LPDIRECT3D8 & GameDX::getD3D() {
    return m_pd3d;
}

D3DRECT & GameDX::getClientSurfaceCoords() {
    static D3DRECT rect = {0, 0, g_winSizeX, g_winSizeY};
    return rect;
}

LPD3DXFONT & GameDX::getFont() {
    return m_pd3dFont;
}


LPDIRECTINPUT8 & GameDX::getDirectInput() {
    return m_pdi;
}

using namespace std;

void GameDX::registerListener( GameDXListener& listener ) {
    LVectorIter iter = find( listeners.begin(), listeners.end(), &listener );
    if ( iter == listeners.end() )
        listeners.push_back( &listener );
}

void GameDX::removeListener( GameDXListener& listener ) {
    LVectorIter iter = find( listeners.begin(), listeners.end(), &listener );
    if ( iter != listeners.end() )
        listeners.erase( iter );
}

void GameDX::generateLostEvent() {
    LVectorIter iter = listeners.begin();
    for ( ; iter != listeners.end(); ++iter ) {
        (*iter)->OnLostDevice();
    }
}

void GameDX::generateResetEvent() {
    LVectorIter iter = listeners.begin();
    for ( ; iter != listeners.end(); ++iter ) {
        (*iter)->OnResetDevice();
    }
}

HRESULT GameDX::setupPresentParams( bool isWindowed ) {
    D3DDISPLAYMODE d3ddm;
    HRESULT r = m_pd3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);
    if( FAILED(r) ) {
        TRACE("Could Not Get Display Adapter Information");
        return r;
    }

    //create display parameters for device "creation"
    D3DFORMAT FullScreenFormat = D3DFMT_A8R8G8B8;

    ZeroMemory( &d3dpp, sizeof(D3DPRESENT_PARAMETERS) );
    d3dpp.Windowed = isWindowed;
    //tell DirectX that it is allowed to mess with the back buffer
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    //set the back buffer to the same format as the primary surface
    d3dpp.BackBufferFormat = isWindowed ? d3ddm.Format : FullScreenFormat;
    d3dpp.BackBufferWidth = isWindowed ? g_winSizeX : DEFAULT_WINDOW_WIDTH;
    d3dpp.BackBufferHeight = isWindowed ? g_winSizeY : DEFAULT_WINDOW_HEIGHT;
    d3dpp.BackBufferCount = 1;
    //set target window to render to
    d3dpp.hDeviceWindow = g_win->getHwnd();
    //turn on the depth buffer
    d3dpp.EnableAutoDepthStencil = FALSE;
    //d3dpp.AutoDepthStencilFormat = D3DFMT_D16; //16bit depth buffer
    //set the refresh rate - not used if windowed
    d3dpp.FullScreen_RefreshRateInHz = isWindowed ? 0 : 60;
    //set the presentation interval - MUST be 0 if windowed
    // Use interval one to setup vsynch in full screen mode
    d3dpp.FullScreen_PresentationInterval = isWindowed ? 0 : D3DPRESENT_INTERVAL_ONE;
    //allow access to the back buffer directly for 2D work (slow...)
    d3dpp.Flags = D3DPRESENTFLAG_LOCKABLE_BACKBUFFER;

    return S_OK;
}
