/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _HOMINGBOMB_H_
#define _HOMINGBOMB_H_

#include "IsoBomb.h"
#include "Bomb.h"
#include "Character.h"
#include "AStar.h"
#include "SoundSystem.h"

/**
 * A bomb that follows a specified target and explodes upon contact.
 */
class HomingBomb : public Bomb {

public:

    /**
     * Default constructor
     */
    HomingBomb();

    /**
     * Constructor
     *
     * @param initPos               the initial position on the map
     * @param initDir               the initial direction
     * @param size                  the size of the bomb's explosion
     * @param delay                 the amount of time before the bomb detonates
     * @param initSpeed             the speed at which the bomb moves
     * @param c                     the character to target
     */
    HomingBomb( const Rect& initPos, int initDir, int size, 
                float delay, float initSpeed, const Character* c );

    /**
     * Destructor
     */
    virtual ~HomingBomb();

    /**
     * Update the bomb's logic.
     *
     * @param dt                        the amount of time since the last frame
     * @returns                         true if this bomb has blown up
     */
    virtual bool update( float dt );

    /**
     * Draw the bomb on the screen using the specified anchor point.
     *
     * @param anchor                    the bomb's anchor point
     */
    virtual void draw( const Point& anchor );

private:

    /**
     * Move the bomb towards its destination.
     *
     * @param dt                        the amount of time since the last frame
     */
    void move( float dt );

    /**
     * Returns true if the bomb has reached its destination.
     *
     * @returns                         true if the bomb has reached its destination
     */
    bool hasReachedDestination();

    /**
     * Sets the next destination point and calculates what direction the bomb
     * should move to get there.
     *
     * @returns                         true if another destination was found
     */
    bool getNextDestination();

    /**
     * Forces the bomb to the center of it's current tile in whatever axis
     * it is not walking in.
     */
    void keepInCenter();

private:

    /**
     * The character to target
     */
    const Character* target;

    /**
     * The position of the target the last time a path was found
     */
    Rect targetPos;

    /**
     * How fast this bomb travels
     */
    float speed;

    /**
     * Finds a path to the target
     */
    AStar finder;

    /**
     * The current destination point
     */
    Point destPoint;

    /**
     * The direction this bomb is currently traveling
     */
    int direction;

    /**
     * Animation for the feet
     */
    Animation feetAnim;

    /**
     * Flag to indicate if the bomb has found its path yet.
     */
    bool hasPath;

    /**
     * Is this the first time the Bomb is finding its path?
     */
    bool firstTime;

    /**
     * The sound the bomb is making, if any.
     */
    SoundToken sound;
};

#endif