/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef MAP_H_34ZCF
#define MAP_H_34ZCF

#include "IsoBomb.h"
struct Tile;
class TileLocation;
class Entity;
class Drawable;
class Point;
class Point3D;
class Powerup;

/**
 * The game map can load itself from a file and draw itself.
 *
 * A Map in Super IsoBomb consists of a rectangular grid of tiles each 1.0f
 * wide and 1.0f tall.  Specifing world coordinates in this way makes
 * interpreting values easier, eliminates multiplations, and provides a very
 * easy conversion from world coordinates into integer tile coordinates.
 *
 * A Map can also render itself and all of the objects onto a screen.  It
 * supports two camera targets for split screen.
 */
class Map {
public: //Flags
    /**
     * Flag for the first camera.
     */
    static const int CAMERA0;

    /**
     * Flag for the second camera.
     */
    static const int CAMERA1;

public:

    /**
     * Constructor
     */
    Map();

    /**
     * Destructor
     */
    ~Map();

    /**
     * Load a map file
     *
     * @param fn                the filename of the map file
     * @returns                 true if the file was loaded succesfully
     */
    bool load( const char *fn );


    /**
     * Draws the map around the specified point.  Must be called between a
     * Begin/End scene pair.  All entities added to the render list are
     * rendered with Entity::draw if their tile is on the screen.  This
     * version of draw ignores the camera target that was set with
     * setCameraTarget.
     *
     * @param pt                the point to center the map about
     * @param scrWidth          the width of the view window
     * @param scrHeight         the height of the view window
     */
    void draw( const Point &pt, int scrWidth, int scrHeight );

    /**
     * Draws the map around the specified Entity target that was set in the
     * setCameraTarget method.  If a target is not set, the map is centered
     * on the point 0, 0.
     *
     * The cameraFlags parameter is constructed by bitwise ORing the flags
     * for the cameras you want to show (CAMERA0, CAMERA1).  If more than one
     * camera is displayed, the given area is split up into different viewports.
     *
     * @see draw( const Point&, int, int )
     *
     * @param scrWidth          the width of the view window
     * @param scrHeight         the height of the view window
     * @param cameraFlags       which cameras to show
     */
    void draw( int scrWidth, int scrHeight, int cameraFlags );

    /**
     * Returns the number of cameras for this Map.  Currently returns 2.
     */
    int getNumCameras() const;

    /**
     * Sets the map's camera to look at the given target.  Every time the
     * renderer is called with the draw method, Map examines this Entity's
     * position as information to center the view about.
     *
     * You can pass NULL to the parameter target to unset the current camera
     * target.  A camera must have a target set in order to be rendered.
     *
     * Map has several cameras (2 at this time).  The camera parameter
     * denotes which camera to set the target for.  The first camera is camera
     * 0, the second 1, and so on.
     */
    void setCameraTarget( const Entity* target, int camera );

    /**
     * Returns the tile at the supplied point, which must be in the map (passes
     * the !isLocationOutside test).
     *
     * @param pt                the point of the desired Tile
     * @returns                 the Tile found
     */
    const Tile* getTileAt( const Point &pt ) const;

    /**
     * Returns true if the given TileLocation is not on the map -- meaning it
     * is not a valid tile coordinate.
     */
    bool isLocationOutside( const TileLocation& loc ) const;

    /**
     * Returns true if the given Point is not on the map -- meaning it
     * is not a valid tile coordinate.
     */
    bool isLocationOutside( const Point& pt ) const;

    /**
     * Returns true if the given Point is not on the map -- meaning it
     * is not a valid tile coordinate.
     */
    bool isLocationOutside( const Point3D& pt ) const;

    /**
     * Returns true if any point of the given Rect is off the map.
     */
    bool isLocationOutside( const Rect& r ) const;

    /**
     * Returns the highest height on the map in the given Rect.  The Rect's
     * width and height must both be < 1.0 for this method to work properly.
     */
    int getRectHeight( const Rect& r ) const;

    /**
     * Returns the height of the tile specified by the given point.  The given
     * point must be on the map (therefore it should pass the isLocationOutside
     * test before this call.
     */
    int getPointHeight( const Point& p ) const;

    /**
     * Returns the height of the tile specified by the given point.  The given
     * point must be on the map (therefore it should pass the isLocationOutside
     * test before this call.
     */
    int getPointHeight( const Point3D& p ) const;

    /**
     * Adds the given object to the rendering list.  The given object must be
     * inside of the map coordinates.  The parameter "obj" is not modified,
     * but must be mutable since the Drawable::draw method is allowed to be
     * mutable.
     *
     * @param obj the object to add.
     */
    void addToRenderList( Drawable& obj );

    /**
     * Clears all Entities that were added to the render list.  An Entity
     * must be removed from the list before it can change map location.
     */
    void clearRenderList();

    /**
     * Given the point in map coordinates, returns the screen coordinate
     * offset from the given startX and startY.
     *
     * This implements isometric projection (2:1 ratio).
     */
    static Point getScreenTileOffset( const Point& pt, float startX, float startY );

    /**
     * Returns the length of the map.
     *
     * @returns                 the length of the map
     */
    int getMapLength() const;

    /**
     * Returns the width of the map.
     *
     * @returns                 the width of the map
     */
    int getMapWidth() const;

    /**
     * Randomly selects one of the starting points specified in the map.
     * Removes the selected point from the list to ensure that the same
     * point isn't picked for both players.
     *
     * @returns                 a starting point
     */
    Point getStartingPoint();

    /**
     * Adds the powerups in this map to the game.
     */
    void addPowerups();

private:

    /**
     * Renders the map.  Expects the viewport to be already set, thus scrWidth
     * and scrHeight are the viewport's width and height.  xShift is set to
     * left side of the viewport.  The xShift is needed because we are sending
     * pre-transformed vertices to the video card, so the viewport's projection
     * has no effect besides clipping.
     */
    void render( const Point &pt, int scrWidth, int scrHeight, int xShift );

    /**
     * Used internally by the load function.  Sets up the tile at the specified
     * position.
     *
     * @param row               the map row
     * @param col               the map column
     * @param height            the height of the tile
     * @param topTex            the id of the texture to use on top of the tile
     * @param sideTex           the id of the texture to use on the sides of the tile
     */
    void setTileAt( int row, int col, int height, int topTex, int sideTex );

private:
    /**
     * Vertex struct used to create the tile triangle strips.  The FVF flags
     * at this time is defined in the Map.cpp file.
     */
    struct TileVertex {
        float x, y, z;
        float rhw;
        float u, v;
    };

    /**
     * Draws a single tile given the center of the tile on the screen (screenX
     * and screenY), and the tile location (mapX and mapY).
     *
     * The xShift is needed for unprojected vertices to show up properly in
     * split screen mode.
     */
    void drawTile( int mapX, int mapY, int screenX, int screenY, int xShift );

    /**
     * Draws a bounding box -- a white wireframe, for the given rect in world
     * coordinates.
     */
    void drawBoundingBox( const Rect& r, int mapX, int mapY, int screenX, int screenY );

    /**
     * Draws the entities on the given tile using the given screen location
     * for the center of the tile.
     *
     * The xShift is needed for unprojected vertices to show up properly in
     * split screen mode.
     */
    void drawEntitiesOnTile( int mapX, int mapY, int screenX, int screenY, int xShift );

    /**
     * Temporary vertex array used in the draw method.  This array is a class
     * member because the rhw values in this array remain constant.  The 5th
     * vertex is only used for bounding box drawing.
     */
    TileVertex vertex[5];

    /**
     * Another temporary vertex buffer for drawing the sides of tiles.
     */
    TileVertex sideVertex[6];

    /**
     * The width of the map (max x value + 1).
     */
    int mapWidth;

    /**
     * The length of the map (max y value + 1)
     */
    int mapLength;

    /**
     * The map tiles
     */
    Tile** tiles;

    /**
     * The camera targets, if any.
     */
    const Entity* cameraTarget[2];

    /**
     * Vector containing starting points.
     */
    std::vector<Point> startPoints;

    /**
     * List containg powerups
     */
    std::list<Powerup*> powerups;

};

#endif
