/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef PARTICLE_H_UMW285
#define PARTICLE_H_UMW285

#include "IsoBomb.h"
#include "Effect.h"
#include "Point3D.h"
#include "Rect.h"
#include "Vector3D.h"

/**
 * A Particle is a timed Effect that is affected by gravity.
 */
class Particle : public Effect {
public:
    /**
     * Constructor
     *
     * @param initPos  the initial position
     * @param initVect the initial vector
     * @param life     how long this Particle lives
     */
    Particle( const Point3D& initPos, const Vector3D& initVect, float life );

    /**
     * Destructor
     */
    virtual ~Particle();

    /**
     * Initializes all textures used for the Particle class.  Throws an
     * IsoException if initialization fails.
     */
    static void initTextures( GraphicsDevice dev );

    /**
     * Uninitalizes the work done by initTextures.  If initTextures threw an
     * exception, releaseTextures need not be called.
     */
    static void releaseTextures();

    /**
     * Sets the color for this Particle.
     */
    void setColor( DWORD color );

    /**
     * Sets the size of this particle.  The size should be a whole number.
     */
    void setSize( float size );

    /**
     * Sets the bounce factor, a number (sane values from 0.0 to 1.0, legal
     * values are >= 0.0) of how much force is rebounded back to the Particle
     * when it bounces.
     */
    void setBounceFactor( float fact );

    virtual bool update( float dt );

    virtual void draw( const Point& anchor );

    virtual float getHeight() const;

    virtual bool needsTransform() const;

    virtual const Rect& getPosition() const;

private:
    /**
     * Vertex struct used to create the tile triangle strips.  The FVF flags
     * at this time is defined in the Map.cpp file.
     */
    struct ParticleVertex {
        float x, y, z;
        float rhw;
        DWORD color;
        float u, v;
    };

    static ParticleVertex v[4];

    /**
     * Particle is not defaultly constructable.
     */
    Particle();

    float life;

    Point3D pos;

    /**
     * Movement vector that describes how this Particle is moving.
     */
    Vector3D vect;

    /**
     * A Rect representation of the actual position pos, that is cached for
     * the getPosition method.
     */
    Rect cached;

    /**
     * This Particle's color.
     */
    DWORD color;

    /**
     * This Particle's size.
     */
    float size;

    /**
     * This Particle's bounce factor.
     */
    float bounceF;

    static Texture tex;
};

#endif
