/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _POWERUP_H_
#define _POWERUP_H_

#include "IsoBomb.h"
#include "Entity.h"
#include "CharacterProps.h"
#include "Character.h"

// Powerup types
const char HP_UP =          'H';
const char SPEED_UP =       'D';
const char BOMBS_UP =       'B';
const char BLAST_UP =       'E';
const char THROW_UP =       'T';
const char HOMING_UP =      'G';

/**
 * Modifies Character's properties when they collide.
 *
 * Powerup is not defaultly-constructable or copyable.
 */
class Powerup : public Entity {

public:

    /**
     * Initializes the respawn time from the given ConfigFile.
     */
    static void initVars( ConfigFile& f );

    /**
     * Constructor
     *
     * @param initPos               initial position on the map
     * @param initHeight            the height of this powerup on the map.  Should
     *    match result of getRectHeight( initPos ) from Map.
     * @param powType                  type of powerup
     */
    Powerup( const Rect& initPos, float height, char powType );

    /**
     * Destructor
     */
    virtual ~Powerup();

    /**
     * Update the game logic.
     *
     * @param dt                the amount of time since the last frame
     * @returns                 true if this entity should be removed from the game
     */
    virtual bool update( float dt );

    /**
     * Draw the powerup
     *
     * @param anchor            where to attach
     */
    virtual void draw( const Point& anchor );
    
    /**
     * Affects a Character with this powerup's properties
     *
     * @param c                character to affect
     */
    void affectCharacter( Character* c );

    /**
     * Returns the height of the powerup on the map.
     *
     * @returns                the height of the powerup on the map
     */
    virtual float getHeight() const;

    /**
     * Returns the type of powerup.
     */
    char getPowerupType() const;

private:
    Powerup();
    Powerup( const Powerup& o );
    Powerup& operator = ( const Powerup& rhs );

    /**
     * The time it takes for a Powerup to respawn.
     */
    static float respawnTime;

    /**
     * Probability of contracting a virus
     */
    static int virusProb;

    /**
     *	Used as a measure of "total" time for use in the sin wave.
     */
    float sinCount;

private:
    /**
     * The type of Powerup, used for self-copying.
     */
    char type;

    /**
     * The powerups properties that will be applied
     */
    CharacterProps props;

    /**
     * A flag indicating whether this powerup has been used
     */
    bool usedUp;

    /**
     * Flag indicating this powerup contains a virus
     */
    bool isVirus;

};

#endif